import requests
from bs4 import BeautifulSoup


class PostGetter:
    def __init__(self):
        url = "https://khuyentran1401.github.io/Python-data-science-code-snippet/"
        self.soup = self.read_html(url)

    def read_html(self, url: str):
        html = requests.get(url).text
        soup = BeautifulSoup(html, "html.parser")
        return soup

    @staticmethod
    def get_titles(rows: list):
        return [row.text for row in rows if not row.a and row.text != "\xa0"]

    @staticmethod
    def get_link(row_with_link):
        if row_with_link.text == "\xa0":
            return ""
        return row_with_link.a.attrs["href"]

    @classmethod
    def get_links(cls, rows: list):
        links = [row for row in rows if row.a or row.text == "\xa0"]
        post_links = [
            cls.get_link(link)
            for link in links
            if "mathdatasimplified" in cls.get_link(link)
        ]
        code_links = [
            cls.get_link(link)
            for link in links
            if "mathdatasimplified" not in cls.get_link(link)
        ]
        return post_links, code_links

    @classmethod
    def get_titles_with_links(cls, rows: list):
        titles = cls.get_titles(rows)
        post_links, code_links = cls.get_links(rows)
        titles_to_post = dict(zip(titles, post_links))
        titles_to_code = dict(zip(titles, code_links))
        return titles_to_post, titles_to_code


class AllPostGetter(PostGetter):
    def __init__(self):
        super().__init__()

    def get_all_titles_with_links(self):
        rows = self.soup.find_all("td")
        return self.get_titles_with_links(rows)


class CategoryGetter(PostGetter):
    def __init__(self, category: str):
        super().__init__()
        self.category = category

    def category_to_id(self):
        return "-".join(self.category.lower().split(" "))

    def get_category(self):
        category = self.category_to_id()
        if category == "numpy":
            return self.soup.find("h1", id=category)
        return self.soup.find("h3", id=category)

    def get_table(self):
        category = self.get_category()
        table = category.next_sibling.next_sibling
        return table

    def get_titles_with_links_for_category(self):
        table = self.get_table()
        rows = table.find_all("td")
        titles_to_post, titles_to_code = self.get_titles_with_links(rows)
        return titles_to_post, titles_to_code



import requests
from rich.console import Console
from rich.syntax import Syntax

console = Console()


def get_raw_url(url: str):
    url = url.replace(
        "https://github.com/", "https://raw.githubusercontent.com/"
    )
    url = url.replace("/blob", "")
    url = url.replace("/tree", "")
    return url


def get_file_name(url: str):
    return url.split("/")[-1]


def get_extension(filename: str):
    if filename.endswith(".sh"):
        return "bash"
    else:
        return "python"


def print_code(url: str, filename: str):
    raw_url = get_raw_url(url)
    code = requests.get(raw_url).text

    console.print("Here is the code for you to copy: :books: \n")
    console.print(Syntax(code, get_extension(filename)))
    console.print("\n")
    return code



import questionary
import typer

from .get_contents import AllPostGetter
from .question import app
from .utils import (console, get_choices, get_code, get_post_from_questionary,
                    handle_exception)


@handle_exception
def get_post_from_question(titles: list):
    return questionary.select(
        "Which post do you want to choose", choices=titles
    ).ask()


@app.command()
def search_posts(pattern=typer.Argument(..., help="String pattern")):
    """Search post based on the string pattern"""
    (
        titles_to_post,
        titles_to_code,
    ) = AllPostGetter().get_all_titles_with_links()

    relevant_titles = get_titles_based_on_pattern(
        pattern, titles_to_post.keys()
    )

    title = get_post_from_questionary(get_choices(relevant_titles))
    post = titles_to_post[title]
    console.print(f"Here is the link: [bold cyan]{post}[/bold cyan] :boom:")
    get_code(titles_to_code[title])


def is_relevant_title(pattern: str, title: str):
    return pattern.lower() in title.lower()


@handle_exception
def get_titles_based_on_pattern(pattern: str, titles: list):
    relevant_titles = [
        title for title in titles if is_relevant_title(pattern, title)
    ]
    return relevant_titles



import questionary
import typer

from .get_contents import CategoryGetter
from .utils import (CATEGORY_TO_SUBCATEGORY, console, get_choices, get_code,
                    get_post_from_questionary, handle_exception)

app = typer.Typer(
    name="python_snippet",
    help="Command-line tool to search for Python and data science snippets at Data Science Simplified",
)


@handle_exception
def get_category_from_questionary():
    return questionary.select(
        "Which category do you want to choose",
        choices=[
            "Python",
            "Pandas",
            "NumPy",
            "Data Science Tools",
            "Terminal",
            "Cool Tools",
            "Jupyter Notebook",
        ],
    ).ask()


@handle_exception
def get_subcategory_from_questionary(subcategories: list):
    return questionary.select(
        "Which subcategory do you want to choose",
        choices=subcategories,
    ).ask()


def get_subcategory(category: str):
    subcategories = CATEGORY_TO_SUBCATEGORY[category]
    if subcategories:
        subcategory = get_subcategory_from_questionary(subcategories)
    else:
        subcategory = category
    return subcategory


def get_post(subcategory: str):

    titles_to_post, titles_to_code = CategoryGetter(
        subcategory
    ).get_titles_with_links_for_category()

    choices = get_choices(titles_to_post.keys())

    title = get_post_from_questionary(choices)

    post = titles_to_post[title]
    console.print(f"Here is the link: [bold cyan]{post}[/bold cyan] :boom:")
    return titles_to_code[title]


@app.command()
def search_category():
    """Select post based on category"""

    category = get_category_from_questionary()
    subcategory = get_subcategory(category)
    code_link = get_post(subcategory)
    get_code(code_link)



import sys

import questionary
from decorator import decorator
from questionary import Choice
from rich.console import Console

from .download_code import get_file_name, print_code

console = Console()

CATEGORY_TO_SUBCATEGORY = {
    "Python": [
        "Number",
        "Boolean",
        "String",
        "List",
        "Tuple",
        "Dictionary",
        "Datetime",
        "Function",
        "Classes",
        "Files",
        "Error handling",
        "Interact with terminal",
        "Best practices",
        "Code speed",
    ],
    "Pandas": ["Change Values", "Get Values", "Testing"],
    "NumPy": [],
    "Data Science Tools": [
        "Testing",
        "Data",
        "Feature extraction",
        "Visualization",
        "Sharing and downloading",
        "Natural language processing",
        "Tools for best Python practices",
        "Speed up code",
        "Better Pandas",
        "Machine learning",
    ],
    "Terminal": [
        "Text",
        "Files",
        "Tracking",
        "Python",
        "Prettify Terminal",
        "Sharing",
        "Productive hacks",
    ],
    "Cool tools": [
        "Better output",
        "Tracking",
        "Data",
        "Automation",
        "Git and GitHub",
        "Alternative Approach",
    ],
    "Jupyter Notebook": [],
}


@decorator
def handle_exception(func, *args, **kwargs):
    try:
        val = func(*args, **kwargs)
        if val is None:
            sys.exit(1)
        return val
    except KeyboardInterrupt:
        console.print("Cancelled by user")
    except ValueError:
        console.print("No results found")
        sys.exit(1)


@handle_exception
def get_post_from_questionary(choices: list):
    return questionary.select(
        "Which post do you want to choose?", choices=choices
    ).ask()


@handle_exception
def get_save_code_from_questionary():
    return questionary.select(
        "Would you like to save the code snippet?", choices=["Yes", "No"]
    ).ask()


def get_code(code_link: str):
    if code_link == "":
        return

    file_name = get_file_name(code_link)
    if file_name.endswith(".ipynb"):
        console.print(
            "This is a notebook so the source code will not be printed on the terminal."
        )
    else:
        code = print_code(code_link, file_name)

    save_code = get_save_code_from_questionary()

    if save_code == "Yes":
        with open(file_name, "w") as f:
            f.write(code)
        console.print(
            f"The file {file_name} is saved to your current directory :sparkles:"
        )


def get_color(num: int):
    if num % 2 == 0:
        return "fg:#809bce bold"
    return "fg:#d6eadf"


def get_choices(titles: list):
    return [
        Choice(title=[(get_color(i), choice)])
        for i, choice in enumerate(titles)
    ]



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['python_snippet']

package_data = \
{'': ['*']}

install_requires = \
['bs4>=0.0.1,<0.0.2',
 'decorator>=5.0.9,<6.0.0',
 'questionary>=1.9.0,<2.0.0',
 'requests>=2.25.1,<3.0.0',
 'rich>=10.4.0,<11.0.0',
 'textdistance>=4.2.1,<5.0.0',
 'typer-cli>=0.0.11,<0.0.12',
 'typer>=0.3.2,<0.4.0']

entry_points = \
{'console_scripts': ['python-snippet = python_snippet.main:app']}

setup_kwargs = {
    'name': 'python-snippet',
    'version': '0.1.5.6',
    'description': 'Python and Data Science Snippets on the command line',
    'long_description': '# Python Snippet Tool\n\nA tool to get Python and data science snippets at [Data Science Simplified](https://mathdatasimplified.com/) on the command line. \n\n## Installation\n```bash\npython -m pip install --user python-snippet\n```\n**Note**: _The `--user` is important. It ensures you install it in your directory and not in the global system. `python -m` ensures that you install python_snippet in the default python version._\n\nMake sure your default Python version is >= 3.6.2.\n```bash\npython --version\n```\n\n\n## Usage\n### Search for posts based on category\n![gif](https://github.com/khuyentran1401/python_snippet/blob/master/images/search_category.gif?raw=true)\nYou can search for posts based on category by typing:\n```bash\npython-snippet search-category\n```\nOnce you select a post, you will receive an output like below:\n![image](https://github.com/khuyentran1401/python_snippet/blob/master/images/search_category.png?raw=True)\n### Search for posts based on a string pattern\n![gif](https://github.com/khuyentran1401/python_snippet/blob/master/images/search_posts.gif?raw=true)\nYou can also search for posts using a string pattern such as `pandas`\n```bash\npython-snippet search-posts pandas\n```\n![image](https://github.com/khuyentran1401/python_snippet/blob/master/images/search_posts_pandas.png?raw=true)\n',
    'author': 'khuyentran1401',
    'author_email': 'khuyentran1476@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/khuyentran1401/python_snippet.git',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.6.2,<3.10',
}


setup(**setup_kwargs)
